'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _scms = require('./scms');

var _scms2 = _interopRequireDefault(_scms);

var _processFiles = require('./processFiles');

var _processFiles2 = _interopRequireDefault(_processFiles);

var _createIgnorer = require('./createIgnorer');

var _createIgnorer2 = _interopRequireDefault(_createIgnorer);

var _createMatcher = require('./createMatcher');

var _createMatcher2 = _interopRequireDefault(_createMatcher);

var _isSupportedExtension = require('./isSupportedExtension');

var _isSupportedExtension2 = _interopRequireDefault(_isSupportedExtension);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = (currentDirectory, {
  config,
  since,
  staged,
  pattern,
  restage = true,
  branch,
  bail,
  check,
  verbose,
  onFoundSinceRevision,
  onFoundChangedFiles,
  onPartiallyStagedFile,
  onExamineFile,
  onCheckFile,
  onWriteFile
} = {}) => {
  const scm = (0, _scms2.default)(currentDirectory);
  if (!scm) {
    throw new Error('Unable to detect a source control manager.');
  }
  const directory = scm.rootDirectory;

  const revision = since || scm.getSinceRevision(directory, { staged, branch });

  onFoundSinceRevision && onFoundSinceRevision(scm.name, revision);

  const rootIgnorer = (0, _createIgnorer2.default)(directory);
  const cwdIgnorer = currentDirectory !== directory ? (0, _createIgnorer2.default)(currentDirectory) : () => true;

  const changedFiles = scm.getChangedFiles(directory, revision, staged).filter(_isSupportedExtension2.default).filter((0, _createMatcher2.default)(pattern)).filter(rootIgnorer).filter(cwdIgnorer);

  const unstagedFiles = staged ? scm.getUnstagedChangedFiles(directory, revision).filter(_isSupportedExtension2.default).filter((0, _createMatcher2.default)(pattern)).filter(rootIgnorer).filter(cwdIgnorer) : [];

  const wasFullyStaged = f => unstagedFiles.indexOf(f) < 0;

  onFoundChangedFiles && onFoundChangedFiles(changedFiles);

  const failReasons = new Set();

  (0, _processFiles2.default)(directory, changedFiles, {
    check,
    config,
    onWriteFile: file => {
      onWriteFile && onWriteFile(file);
      if (bail) {
        failReasons.add('BAIL_ON_WRITE');
      }
      if (staged && restage) {
        if (wasFullyStaged(file)) {
          scm.stageFile(directory, file);
        } else {
          onPartiallyStagedFile && onPartiallyStagedFile(file);
          failReasons.add('PARTIALLY_STAGED_FILE');
        }
      }
    },
    onCheckFile: (file, isFormatted) => {
      onCheckFile && onCheckFile(file, isFormatted);
      if (!isFormatted) {
        failReasons.add('CHECK_FAILED');
      }
    },
    onExamineFile: verbose && onExamineFile
  });

  return {
    success: failReasons.size === 0,
    errors: Array.from(failReasons)
  };
};